@@ -4,16 +4,42 @@
 ** modify it under the terms of GNU Lesser General Public License.
 */
 
-/* recoding this in a more portable way is left as an exercise to the reader :p */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <malloc.h>
+#ifdef _WIN32
+#include <windows.h>
+#include <limits.h>
+#else
 #include <sys/time.h>
+#endif
+
+#include "iksemel.h"
 
 /* timing functions */
 
+#ifdef _WIN32
+static DWORD start_tv;
+
+void
+t_reset (void)
+{
+	start_tv = GetTickCount ();
+}
+
+unsigned long
+t_elapsed (void)
+{
+	DWORD end_tv;
+
+	end_tv = GetTickCount ();
+	if (end_tv < start_tv)
+		return UINT_MAX - (start_tv - end_tv - 1);
+	else
+		return end_tv - start_tv;
+}
+
+#else
 static struct timeval start_tv;
 
 void
@@ -33,23 +59,26 @@ t_elapsed (void)
 	msec -= (start_tv.tv_sec * 1000) + (start_tv.tv_usec / 1000);
 	return msec;
 }
+#endif
 
 /* memory functions */
 
-int
-m_used (void)
+static void *
+m_malloc (size_t size)
 {
-	struct mallinfo mi;
-
-	mi = mallinfo ();
-	return mi.uordblks;
+	void *ptr = malloc (size);
+	printf ("MEM: malloc (%d) => %p\n", size, ptr);
+	return ptr;
 }
 
-int
-m_arena (void)
+static void
+m_free (void *ptr)
 {
-	struct mallinfo mi;
+	printf ("MEM: free (%p)\n", ptr);
+}
 
-	mi = mallinfo ();
-	return mi.arena;
+void
+m_trace (void)
+{
+	iks_set_mem_funcs (m_malloc, m_free);
 }
