@@ -6,5 +6,5 @@
 
 void t_reset (void);
 unsigned long t_elapsed (void);
-int m_used (void);
-int m_arena (void);
+
+void m_trace (void);
